#include <stdio.h>
#include <netdb.h> 
#include <sys/types.h>
#include <fcntl.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <sys/socket.h>

#ifdef DVE
#include "sessionData.h"
#endif

#define DAEMON_PORT 3579
#define LAUNCH_PORT 3580

int remote;
int numMem;
int startFlag;
char *information;
char *msg;
char *localURL;


/* launch.c */

int setupSocket(int port);
void setSocketNonBlocking(int socket);
int connectToRemoteDaemon(int sock, char * name);
#ifdef DVE
int launchRemote(DveConfMgrUserAddr *list, int n);
#endif
char* readConfDataFromStream(char *p, int *num, int *tag);
void parseIPaddress(char *IP, char *name, char *host);
int findproc(char *proc);
void failure(char *msg);
int check_process(char *name, const char* opt);
char* changeURL(char *msg);

int setupSocket(int port)
{
  int sock;
  int optval;
  struct sockaddr_in name;

  if ( ( sock = socket(AF_INET, SOCK_STREAM, 0) ) < 0){
    fprintf(stderr, "Can't create socket.\n");
    return -1;
  }
  
  setsockopt(sock, IPPROTO_TCP, TCP_NODELAY, (char *)&optval,
             sizeof(optval));

  optval=1000;
/*  setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (char *)&optval,
             sizeof(optval)); 
  setsockopt(sock, SOL_SOCKET, SO_REUSEPORT, (char *)&optval,
             sizeof(optval)); 
*/
  name.sin_family = AF_INET;
  name.sin_addr.s_addr = INADDR_ANY;
  name.sin_port = htons(port);
  if ( bind(sock, (struct sockaddr*) &name, sizeof(name) ) < 0){
    perror("Bind failed.\n");
    return -1;
  }
/*  printf("Socket open and bound successfully\n");*/
  return (sock);
} /* setupSocket() */

void setSocketNonBlocking(int socket)
{
  if (fcntl(socket, F_SETFL, O_NDELAY) < 0){
    fprintf(stderr, "Error while setting data socket to non blocking.\n");
  }
  printf("socket non-blocking\n");
} /* setSocketNonBlocking() */

int connectToRemoteDaemon(int sock, char *name)
{
  struct sockaddr_in serv_addr;
  struct hostent *host;

  
/*  printf("Connecting on socket %d to %s len: %d\n", sock, name, strlen(name));*/
  
  if ((host = (struct hostent*) malloc(sizeof(struct hostent)))==NULL)
    {
      printf("Cannot allocate memory!\n");
      return -1;
    }
  bzero((char *) &serv_addr, sizeof(serv_addr));
  
  serv_addr.sin_family = AF_INET;
  serv_addr.sin_port = DAEMON_PORT;

/*  printf("Gethostbyname(%s)\n", name);*/
  if ( ( host = gethostbyname(name) ) == 0)
  {
    perror("Can't get server address by name.\n");
    return -1;
  }

  bcopy(host -> h_addr, (caddr_t) &serv_addr.sin_addr, host->h_length);
  if (connect(sock, (struct sockaddr *) &serv_addr, sizeof(serv_addr) ) < 0)
  {
    printf("Data port can't connect to daemon on %s\n", host->h_name);
    perror("");
    return -1;
  }
/*  printf("connected to remote daemon on %s\n", name);*/
  return sock;
} /* connectToRemoteDaemon() */


#ifdef DVE
int launchRemote(DveConfMgrUserAddr *list, int n)
{
  int i, err;
  int sock;

  mtemp=mbrs;
/*  printf("Number of remote systems: %d\n", n);*/
  if ((sock = setupSocket(LAUNCH_PORT))<0)
  {
    printf("Can not open local socket! Aborting...\n");
    return -1;
  }
  for(i=0;i<n;i++)
  {
/*    printf("Launching on %s ...\n", mtemp->hostName);*/
    if ((err = connectToRemoteDaemon(sock, mtemp->hostName))>0)
    {
      write(sock, localURL, strlen(localURL));
    }
/*    else printf("Can not connect to %s\n", mtemp->hostName);*/
    
    mtemp+=1;/*sizeof(DveConfMgrUserAddr);*/
  }
  close(sock);
  return 0;
}
#endif


char* readConfDataFromStream(char *p, int *num, int *tag)
{
  int i, len;
  char *buffer, *temp;
  char *ptr, *pos;

  ptr=p;
  pos=strchr(ptr, 35); /* pre # */
  ptr=pos+1;
  *tag=(char)(*ptr)-48;
  pos=strchr(ptr, 35); /* post # */
  ptr=pos+1;
  pos=strchr(ptr, 35); /* pre # */
  ptr=pos+1;
  *num=(char)(*ptr)-48;
  pos=strchr(ptr, 35); /* post # */
  ptr=pos+1;
  
  buffer=(char*)malloc((*num)*256);
  temp=buffer;
  if (!buffer)
    {
      fprintf(stdout, "Could not allocate memory for addresses!\n");
      return NULL;
    }

  fflush(stdout);
  for (i=0;i<(*num);i++)
  {
/*    printf("Pointer: %d ", (int)temp);*/
	pos=strchr(ptr, 35); /* pre # */
	ptr=pos+1;
	pos=strchr(ptr, 35); /* post # */
	len=(int)pos-(int)ptr;
/*	printf("l=%d\n", len);*/
	memcpy(temp, ptr, len);
	temp[len]='\0';
	ptr+=len+1;
	temp+=len+1;
  }
  return buffer;
} /* readConfDataFromStream() */

void parseIPaddress(char *IP, char *name, char *host)
{
  char *ptr, *pos;
  int len, i;

  ptr = IP;
  pos = strchr(ptr, 64); /* code of @ */
  len = (int)pos - (int)ptr;
  memcpy(name, ptr, len);
  name[len]='\0';
  ptr += len+1;
  memcpy(host, ptr, strlen(IP)-len-1);
  host[strlen(IP)-len-1]='\0';
/*  printf("parseIP: name: %s host: %s|\n", name, host);*/
} /* parseIPaddress() */


int
findproc(char *proc)
{
  int err, pid, i;
  FILE *fp;
  char command[256];
  char name[9];

  for(i=0;i<8;i++)
    name[i]=proc[i];
  name[8]='\0';
  strcpy(command, "ps -e | grep ");
  strcat(command, name);
  strcat(command, " > .result");

  err = system(command);
  if (err == -1)
  {
    failure("launcher error");
    return err;
  }

  pid=0;
  fp = fopen(".result", "r");
  if (!fp)
  {
    failure("launcher: file open error");
    return -1;
  }
  fscanf(fp, "%d", &pid);
  fclose(fp);
  system("rm .result");
/*  printf("%s pid = %d\n", proc, pid);*/

  return pid;
} /* findproc() */

void
failure(char *msg)
{
  fprintf(stderr, msg);
} /* failure() */

int
check_process(char *name, const char *opt)
{
  int pid;
  int err;
  char command[256];

  command[0]='\0';
  strcat(command, name);
  strcat(command, " ");
  strcat(command, opt);
  strcat(command, " &");

  pid = findproc(name);
  if (pid<0) return -1; /* error */
  if (!pid) {

    err = system(command);
    if (err == -1)
    {
      failure( "system command failure in launcher");
      return err;
    }
  }
  else
  {
/*    fprintf(stdout, "%s running already.\n", name);*/
    err=1971;
  }

  return err;
} /* check_process() */


char* changeURL(char *msg)
{
  char *pos, *newpos, *url;
  int k=0, len, needed;

  pos=msg;
/*  printf("Init pos: %d msg: %d %s\n", (int)pos, (int)msg, msg);*/

  while ((pos=strchr(pos, 47))!=NULL)
  {
    k++;
/*    printf("Pos: %d\n", (int)pos);*/
    newpos=pos;
    pos+=1;
    if (k>10) break;
  }
/*  printf("Last position: %d\n", (int)newpos);*/
  newpos++;
  len=(int)newpos-(int)msg;
  needed=len+strlen("default.dve");
/*  printf("Len: %d needed: %d\n", len, needed);*/
  url = (char*)malloc(needed);
  memcpy(url, msg, len);
  url[len]='\0';
/*  printf("First url: %s\n", url);*/
  strcat(url, "default.dve");
/*  printf("Second url: %s\n", url);*/
  return url;
}






